from robotcloud.api import APIEndPointAuthenticated


class APICallLocationServices(APIEndPointAuthenticated):
    """
        Implement GET methods
    """

    def __init__(self, token: str, project_id: str, location_id: str):
        self.project_id = project_id
        self.location_id = location_id
        super().__init__(token)

    def get_endpoint(self):
        return f'projects/{self.project_id}/locations/{self.location_id}/services'


class APICallLocationServicesInstances(APIEndPointAuthenticated):
    """
        Implement GET, POST methods
    """

    def __init__(self, token: str, project_id: str, location_id: str, service_type: str):
        self.project_id = project_id
        self.location_id = location_id
        self.service_type = service_type
        super().__init__(token)

    def get_endpoint(self):
        return f'projects/{self.project_id}/locations/{self.location_id}/services/{self.service_type}/instances'


class APICallProjectServicesInstances(APIEndPointAuthenticated):
    """
        Implement GET methods
    """

    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f'projects/{self.project_id}/instances'


class APICallProjectServices(APIEndPointAuthenticated):
    """
        Implement GET methods
    """

    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f'projects/{self.project_id}/services'


class APICallProjectServiceInstances(APIEndPointAuthenticated):
    """
        Implement GET, PUT, DELETE methods
    """

    def __init__(self, token: str, project_id: str, service_type: str, instance_id: str = None):
        self.project_id = project_id
        self.service_type = service_type
        self.instance_id = instance_id
        super().__init__(token)

    def get_endpoint(self):
        if self.instance_id is None:
            return f'projects/{self.project_id}/services/{self.service_type}/instances'
        else:
            return f'projects/{self.project_id}/services/{self.service_type}/instances/{self.instance_id}'


class APICallServiceInstanceDeviceConfig(APIEndPointAuthenticated):
    """
        Implement GET, PUT methods
    """

    def __init__(self, token: str, project_id: str, service_type: str, instance_id: str = None):
        self.project_id = project_id
        self.service_type = service_type
        self.instance_id = instance_id
        super().__init__(token)

    def get_endpoint(self):
        return f'projects/{self.project_id}/services/{self.service_type}/instances/{self.instance_id}/deviceconf'


SERVICES_TYPES = [
    {'code': 'PowerMeter_1', 'description': 'Power Meter v1'},
    {'code': 'CoolHeatProd_1', 'description': 'Cool & Heat Producer v1'},
    {'code': 'HeatProd_1', 'description': 'Heat Producer v1'},
    {'code': 'CoolHeatCons_1', 'description': 'Cool & Heat Consumer v1'},
    {'code': 'OutdoorClime_1', 'description': 'OutdoorClime_1'},
    {'code': 'WaterCounter_1', 'description': 'Water Counter v1'},
    {'code': 'RoomClime_1', 'description': 'Room Clime v1'},
    {'code': 'RoomGuestStatus_1', 'description': 'Room Guest Status v1'},
    {'code': 'EnergyProduction_1', 'description': 'Energy Production v1'},
    {'code': 'RoomConsumes_1', 'description': 'Room Consumes v1'},
    {'code': 'EnergyCounter_1', 'description': 'Energy Counter v1'},
    {'code': 'HeatMeter_1', 'description': 'Heat Meter v1'},
    {'code': 'TemporizedOutput_1', 'description': 'Temporized Output v1'},
]


def get_services_types(token):
    return SERVICES_TYPES


def get_location_services(token, project_id, location_id):
    return APICallLocationServices(token, project_id, location_id).get()


def get_location_services_instances(token, project_id, location_id, service_type):
    return APICallLocationServicesInstances(token, project_id, location_id, service_type).get()


def create_location_services_instance(token, project_id, location_id, service_type, data):
    return APICallLocationServicesInstances(token, project_id, location_id, service_type).post(data)


def get_project_services_instances(token, project_id, params=None):
    if params is None:
        params = {}
    return APICallProjectServicesInstances(token, project_id).get(params=params)


def get_project_services(token, project_id):
    return APICallProjectServices(token, project_id).get()


def get_project_service_instances(token, project_id, service_type):
    return APICallProjectServiceInstances(token, project_id, service_type).get()


def get_service_instance_details(token, project_id, service_type, instance_id):
    return APICallProjectServiceInstances(token, project_id, service_type, instance_id).get()


def edit_service_instance(token, project_id, service_type, instance_id, data):
    return APICallProjectServiceInstances(token, project_id, service_type, instance_id).put(data)


def delete_service_instance(token, project_id, service_type, instance_id):
    return APICallProjectServiceInstances(token, project_id, service_type, instance_id).delete()


def get_service_instance_dev_conf(token, project_id, service_type, instance_id):
    return APICallServiceInstanceDeviceConfig(token, project_id, service_type, instance_id).get()


def edit_service_instance_dev_conf(token, project_id, service_type, instance_id, data):
    return APICallServiceInstanceDeviceConfig(token, project_id, service_type, instance_id).put(data)



from abc import ABCMeta

from robotcloud.api import APIEndPointAuthenticated


class APICallServiceInstance(APIEndPointAuthenticated, metaclass=ABCMeta):
    def __init__(self, token: str, project_id: str, service_type: str, instance_id: str = None):
        self.project_id = project_id
        self.service_type = service_type
        self.instance_id = instance_id
        super().__init__(token)


class APICallServiceInstanceData(APICallServiceInstance):
    """
        Implement GET
    """

    def get_endpoint(self):
        if self.instance_id is None:
            return f'projects/{self.project_id}/services/{self.service_type}/data'
        else:
            return f'projects/{self.project_id}/services/{self.service_type}/instances/{self.instance_id}/data'


class APICallServiceInstanceConfig(APICallServiceInstance):
    """
        Implement GET, PUT methods
    """

    def get_endpoint(self):
        if self.instance_id is None:
            return f'projects/{self.project_id}/services/{self.service_type}/configuration'
        else:
            return f'projects/{self.project_id}/services/{self.service_type}/instances/{self.instance_id}/configuration'


class APICallServiceInstanceAlert(APICallServiceInstance):
    """
        Implement GET
    """

    def get_endpoint(self):
        if self.instance_id is None:
            return f'projects/{self.project_id}/services/{self.service_type}/alert'
        else:
            return f'projects/{self.project_id}/services/{self.service_type}/instances/{self.instance_id}/alert'


def get_service_instance_data(token, project_id, service_type, instance_id=None) -> dict:
    """
    If instance_id is None then return data from all project service_type instances
    otherwise, only return the data from the specified instance.

    :param token:
    :param project_id:
    :param service_type:
    :param instance_id:
    :raises:
    :return:
    """
    api_response = APICallServiceInstanceData(token, project_id, service_type, instance_id).get()
    if type(api_response) is dict:
        return api_response  # Single data

    response = {}
    for data in api_response:
        response[data['instance']] = data

    return response


def get_service_instance_config(token, project_id, service_type, instance_id):
    """
    If instance_id is None then return data from all project service_type instances
    otherwise, only return the data from the specified instance.

    :param token:
    :param project_id:
    :param service_type:
    :param instance_id:
    :return:
    """
    return APICallServiceInstanceConfig(token, project_id, service_type, instance_id).get()


def update_service_instance_config(token, project_id, service_type, instance_id, data):
    """
    Update the specified service instance configuration.

    :param token:
    :param project_id:
    :param service_type:
    :param instance_id:
    :param data: Data to update
    :return:
    """
    return APICallServiceInstanceConfig(token, project_id, service_type, instance_id).put(data)


def get_service_instance_alert(token, project_id, service_type, instance_id=None) -> dict:
    """
    If instance_id is None then return data from all project service_type instances
    otherwise, only return the data from the specified instance.

    :param token:
    :param project_id:
    :param service_type:
    :param instance_id:
    :return:
    """
    api_response = APICallServiceInstanceAlert(token, project_id, service_type, instance_id).get()
    if type(api_response) is dict:
        return api_response  # Single data

    response = {}
    for data in api_response:
        response[data['instance']] = data

    return response



from robotcloud.endpoints.locations import *
from robotcloud.endpoints.login import *
from robotcloud.endpoints.organizations import *
from robotcloud.endpoints.projects import *
from robotcloud.endpoints.tags import *
from robotcloud.endpoints.subsystems import *
from robotcloud.endpoints.users import *



from dataclasses import asdict

from robotcloud.api import APIEndPointAuthenticated
from robotcloud.endpoints.organizations import APICallOrganizationUsers
from robotcloud.utils import Pagination


class APICallUser(APIEndPointAuthenticated):
    """
        Implement GET, PUT and DELETE methods
    """
    def __init__(self, token: str, username: str = ''):
        self.username = username
        super().__init__(token)

    def get_endpoint(self):
        return f'users/{self.username}'


class APICallUserProjects(APIEndPointAuthenticated):
    """
        Implement GET, PUT and DELETE methods
    """
    def __init__(self, token: str, username: str, project_id: str = ''):
        self.username = username
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f'users/{self.username}/projects/{self.project_id}'


class APICallUserOrganizations(APIEndPointAuthenticated):
    def __init__(self, token: str, username: str):
        self.username = username
        super().__init__(token)

    def get_endpoint(self):
        return f"users/{self.username}/organizations"


def get_all_users(token, pagination: Pagination = None):
    params = {} if pagination is None else asdict(pagination)
    return APICallUser(token).get(params=params)


def get_user(token, username):
    return APICallUser(token, username).get()


def put_user(token, username, data):
    return APICallUser(token, username).put(data)


def delete_user(token, username):
    return APICallUser(token, username).delete()


def get_user_projects(token, username):
    return APICallUserProjects(token, username).get()


def post_user_project(token, username, data):
    return APICallUserProjects(token, username).post(data)


def update_user_project(token, username, project_id, data):
    return APICallUserProjects(token, username, project_id).put(data)


def delete_user_project(token, username, project_id):
    return APICallUserProjects(token, username, project_id).delete()


def create_user(token, organization_id, data):
    return APICallOrganizationUsers(token, organization_id).post(data)


def get_user_organizations(token, username):
    return APICallUserOrganizations(token, username).get()



from robotcloud.api import APIEndPointAuthenticated


class APICallTag(APIEndPointAuthenticated):
    def __init__(self, token: str, tag_id: str):
        self.tag_id = tag_id
        super().__init__(token)

    def get_endpoint(self):
        return f"tags/{self.tag_id}"


class APICallProjectTags(APIEndPointAuthenticated):
    """
        Implement GET, POST methods
    """
    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f"projects/{self.project_id}/tags"

    def create_tag(self, data):
        return self.post(data)

    def get_tags(self, parent_tag=None, get_all=False):
        query_params = {
            'parent_tag': parent_tag
        }
        if not get_all:
            query_params['no_parent'] = parent_tag is None

        return self.get(params=query_params)


def get_tag(token, tag_id):
    return APICallTag(token, tag_id).get()


def delete_tag(token, tag_id):
    return APICallTag(token, tag_id).delete()


def update_tag(token, tag_id, data):
    return APICallTag(token, tag_id).put(data)



from robotcloud.api import APIEndPointAuthenticated


class APICallLocationDevices(APIEndPointAuthenticated):
    """
        Implement GET, POST methods
    """
    def __init__(self, token: str, location_id: str):
        self.location_id = location_id
        super().__init__(token)

    def get_endpoint(self):
        return f'locations/{self.location_id}/devices'


class APICallProjectDevices(APIEndPointAuthenticated):
    """
        Only implement GET
    """
    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f'projects/{self.project_id}/devices'


class APICallDevices(APIEndPointAuthenticated):
    """
        Implement GET, PUT and DELETE methods
    """
    def __init__(self, token: str, device_id: str):
        self.device_id = device_id
        super().__init__(token)

    def get_endpoint(self):
        return f'devices/{self.device_id}'


class APICallDeviceCompatibleConfigTypes(APIEndPointAuthenticated):
    """
        Implement GET methods
    """
    def __init__(self, token: str, device_id: str):
        self.device_id = device_id
        super().__init__(token)

    def get_endpoint(self):
        return f'devices/{self.device_id}/compatibleconfigurationtypes'


def get_location_devices(token, location_id):
    return APICallLocationDevices(token, location_id).get()


def post_location_device(token, location_id, data):
    """ Create a device in location """
    return APICallLocationDevices(token, location_id).post(data)


def get_project_devices(token, project_id):
    return APICallProjectDevices(token, project_id).get()


def get_device(token, device_id):
    return APICallDevices(token, device_id).get()


def edit_device(token, device_id, data):
    return APICallDevices(token, device_id).put(data)


def delete_device(token, device_id):
    return APICallDevices(token, device_id).delete()


def get_device_compatible_configuration_types(token, device_id):
    return APICallDeviceCompatibleConfigTypes(token, device_id).get()



from robotcloud.constants import SYSTEM_ORGANIZATION_ID
from robotcloud.api import APIEndPointAuthenticated


class APICallOrganization(APIEndPointAuthenticated):
    """
        Implement GET, PUT and DELETE methods
    """
    def __init__(self, token: str, organization_id: str = ''):
        self.organization_id = organization_id
        super().__init__(token)

    def get_endpoint(self):
        return f'organizations/{self.organization_id}'


class APICallOrganizationProjects(APIEndPointAuthenticated):
    def __init__(self, token: str, organization_id: str):
        self.organization_id = organization_id
        super().__init__(token)

    def get_endpoint(self):
        return f"organizations/{self.organization_id}/projects"


class APICallOrganizationUsers(APIEndPointAuthenticated):
    def __init__(self, token: str, organization_id: str):
        self.organization_id = organization_id
        super().__init__(token)

    def get_endpoint(self):
        return f'organizations/{self.organization_id}/users'


def get_organizations(token, with_system: bool = True):
    organizations = APICallOrganization(token).get()
    if not with_system:
        for i, org in enumerate(organizations):
            if org["id"] == SYSTEM_ORGANIZATION_ID:
                del organizations[i]
    return organizations


def create_organization(token, data):
    return APICallOrganization(token).post(data)


def get_organization(token, org_id):
    return APICallOrganization(token, org_id).get()


def update_organization(token, org_id, data):
    return APICallOrganization(token, org_id).put(data)


def delete_organization(token, org_id):
    return APICallOrganization(token, org_id).delete()


def get_organization_projects(token, org_id) -> list:
    return APICallOrganizationProjects(token, org_id).get()


def get_organization_users(token, org_id) -> list:
    return APICallOrganizationUsers(token, org_id).get()



from robotcloud.api import APIEndPointAuthenticated
from robotcloud.endpoints.organizations import APICallOrganizationProjects
from robotcloud.exceptions import BadUsageException


class APICallProjects(APIEndPointAuthenticated):
    def __init__(self, token: str, project_id: str = None):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        if self.project_id is None:
            return "projects/"
        else:
            return f"projects/{self.project_id}"


class APICallProjectUsers(APIEndPointAuthenticated):
    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f"projects/{self.project_id}/users"


class APIEndpointProjectDevices(APIEndPointAuthenticated):
    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f"projects/{self.project_id}/devices"


def get_project(token, project_id):
    return APICallProjects(token, project_id).get()


def get_projects(token):
    return APICallProjects(token).get()


def update_project(token, project_id, new_data):
    return APICallProjects(token, project_id).put(new_data)


def get_project_users(token, project_id):
    return APICallProjectUsers(token, project_id).get()


def get_project_devices(token, project_id):
    return APIEndpointProjectDevices(token, project_id).get()


def create_project(token, data) -> dict:
    if 'organization_id' not in data or not isinstance(data['organization_id'], str):
        raise BadUsageException("")
    org_id = data['organization_id']
    data.pop('organization_id')
    return APICallOrganizationProjects(token, org_id).post(data)


def delete_project(token, project_id) -> dict:
    return APICallProjects(token, project_id).delete()



from dataclasses import dataclass

from robotcloud.utils.datatables.request import DataTableRequest


@dataclass
class Pagination:
    startIndex: int
    maxSize: int

    def __init__(self, req: DataTableRequest):
        self.startIndex = req.start
        self.maxSize = req.length



from abc import ABC, abstractmethod
from base64 import b64encode


class Authenticator(ABC):
    @abstractmethod
    def compute_string(self):
        pass


class BasicAuthenticator(Authenticator):
    def __init__(self, username: str, password: str):
        self.username = username
        self.password = password

    def compute_string(self):
        s = f"{self.username}:{self.password}"
        encoding = "utf-8"
        encoded = s.encode(encoding)
        base64_encoded = b64encode(encoded)
        decoded = base64_encoded.decode(encoding)
        return f"Basic {decoded}"


class BearerAuthenticator(Authenticator):
    def __init__(self, token: str):
        self.token = token

    def compute_string(self):
        return f"Bearer {self.token}"



import os

SYSTEM_ORGANIZATION_ID = os.environ.get("ROBOTCLOUD_SYSTEM_ORGANIZATION_ID ", "org-0")
API_KEY = os.environ.get("ROBOTCLOUD_API_KEY")
ROOT_URL = os.environ.get("ROBOTCLOUD_ROOT_URL")

DEFAULT_TIMEOUT = os.environ.get("ROBOTCLOUD_DEFAULT_TIMEOUT", 5)



import logging
import json

import requests
from abc import abstractmethod
from http import HTTPStatus
from contextlib import suppress

from requests import ConnectionError, Response

import robotcloud.constants
import robotcloud.exceptions as exceptions
from robotcloud.authenticator import (
    Authenticator,
    BasicAuthenticator,
    BearerAuthenticator
)
from robotcloud.constants import DEFAULT_TIMEOUT
from robotcloud.exceptions import RequestConnectionError

log = logging.getLogger(__name__)


def json_from_response(response: Response):
    if response.ok:
        return response.json()

    status_code = int(response.status_code)
    try:
        __print_error(response)
        response_message = response.json()["message"]
    except ValueError:
        response_message = f"Undefined error (code: {status_code})."

    if status_code == HTTPStatus.UNAUTHORIZED:
        raise exceptions.UnauthorizedAccessError(response_message)
    elif status_code == HTTPStatus.FORBIDDEN:
        raise exceptions.ForbiddenError(response_message)
    elif status_code == HTTPStatus.NOT_FOUND:
        raise exceptions.NotFoundError(response_message)
    elif status_code == HTTPStatus.BAD_REQUEST:
        raise exceptions.BadRequestError(response_message)
    elif status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        raise exceptions.InternalServerError(response_message)
    else:
        raise exceptions.RobotcloudRequestError(response_message)


def __print_error(response: Response):
    error = (
        f"\n{'<ERROR: Robotcloud API call>':*^75}"
        f"\n# [{response.request.method}] {response.request.url}"
        f"\n# {response}"
    )

    with suppress(TypeError):
        error += (
            f"\n{json.dumps(response.json(), indent=4)}"
        )

    error += (
        f"\n{'</ERROR: Robotcloud API call>':*^75}\n"
    )
    log.info(error)


class APIRequestHeader:

    def __init__(self, authenticator: Authenticator):
        self.items = {
            "x-api-key": robotcloud.constants.API_KEY,
            'Accept': 'application/json'
        }
        self.authenticator = authenticator

    @property
    def authenticator(self):
        return self._authenticator

    @authenticator.setter
    def authenticator(self, authenticator):
        self._authenticator = authenticator
        self.items["Authorization"] = authenticator.compute_string()


class APIEndPoint:
    ROOT_URL = robotcloud.constants.ROOT_URL

    def __init__(self, headers: dict):
        self.headers = headers

    def get(self, **kwargs):
        try:
            response = requests.request(url=self.url, method="GET", headers=self.headers,
                                        timeout=DEFAULT_TIMEOUT, **kwargs)
        except ConnectionError as e:
            log.exception(f"Get Request Connection error: {self.url}")
            raise RequestConnectionError(str(e))

        return json_from_response(response)

    def post(self, data, **kwargs):
        headers = self.headers.copy()
        headers['Content-Type'] = 'application/json'
        try:
            response = requests.request(url=self.url, method="POST", headers=headers, timeout=DEFAULT_TIMEOUT,
                                        data=json.dumps(data), **kwargs)
        except ConnectionError as e:
            log.exception(f"Post Request Connection error: {self.url}")
            raise RequestConnectionError(str(e))

        return json_from_response(response)

    def put(self, data):
        headers = self.headers.copy()
        headers['Content-Type'] = 'application/json'

        try:
            response = requests.request(url=self.url, method="PUT", headers=headers, timeout=DEFAULT_TIMEOUT,
                                        data=json.dumps(data))
        except ConnectionError as e:
            log.exception(f"Put Request Connection error: {self.url}")
            raise RequestConnectionError(str(e))
        return json_from_response(response)

    def delete(self):
        try:
            response = requests.request(url=self.url, method="DELETE", timeout=DEFAULT_TIMEOUT, headers=self.headers)
        except ConnectionError as e:
            log.exception(f"Delete Request Connection error: {self.url}")
            raise RequestConnectionError(str(e))
        return json_from_response(response)

    @property
    def url(self):
        return self.ROOT_URL + "/" + self.get_endpoint()

    @abstractmethod
    def get_endpoint(self):
        pass


class APIEndPointAuthenticated(APIEndPoint):
    def __init__(self, token: str):
        authenticator = BearerAuthenticator(token)
        self.header = APIRequestHeader(authenticator)
        super().__init__(self.header.items)

    @abstractmethod
    def get_endpoint(self):
        pass


class APIEndPointUnAuthenticated(APIEndPoint):
    def __init__(self, username: str, password: str):
        authenticator = BasicAuthenticator(username, password)
        self.header = APIRequestHeader(authenticator)
        super().__init__(self.header.items)

    @abstractmethod
    def get_endpoint(self):
        pass



class RobotcloudRequestError(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return f'An undefined Robotcloud API response: {self.message}'


class UnauthorizedAccessError(RobotcloudRequestError):

    def __str__(self):
        return f'Unauthorized access: {self.message}'


class NotFoundError(RobotcloudRequestError):

    def __str__(self):
        return 'Not found resource'


class ForbiddenError(RobotcloudRequestError):

    def __str__(self):
        return f'Forbidden access: {self.message}'


class BadRequestError(RobotcloudRequestError):

    def __str__(self):
        return 'Bad request'


class InternalServerError(RobotcloudRequestError):

    def __str__(self):
        return f'Internal server error: {self.message}'


class InvalidServiceNameException(Exception):

    def __str__(self):
        return 'Service name is not valid'


class RequestConnectionError(RobotcloudRequestError):

    def __str__(self):
        return f'Request connection error: {self.message}'


class BadUsageException(RobotcloudRequestError):
    """
    Exception raised when a method is called improperly

    """
    def __str__(self):
        return f'Bad usage exception: {self.message}'



from robotcloud.api import APIEndPointAuthenticated


class APICallProjectSubsystems(APIEndPointAuthenticated):
    def __init__(self, token: str, project_id: str, subsystem_id: str = None):
        self.project_id = project_id
        self.subsystem_id = subsystem_id
        super().__init__(token)

    def get_endpoint(self):
        if self.subsystem_id is None:
            return f"projects/{self.project_id}/subsystems"
        else:
            return f"projects/{self.project_id}/subsystems/{self.subsystem_id}"


def get_project_subsystems(token, project_id) -> list:
    return APICallProjectSubsystems(token, project_id).get()


def create_subsystem(token, project_id, data):
    return APICallProjectSubsystems(token, project_id).post(data)


def get_subsystem_details(token, project_id, subsystem_id):
    return APICallProjectSubsystems(token, project_id, subsystem_id).get()


def edit_subsystem(token, project_id, subsystem_id, data):
    return APICallProjectSubsystems(token, project_id, subsystem_id).put(data)


def delete_subsystem(token, project_id, subsystem_id):
    return APICallProjectSubsystems(token, project_id, subsystem_id).delete()



from robotcloud.api import APIEndPointAuthenticated


class APICallProjectConfigTypes(APIEndPointAuthenticated):
    """
        Implement GET, POST methods
    """
    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f'projects/{self.project_id}/configurationtypes'


class APICallConfigType(APIEndPointAuthenticated):
    """
        Implement GET, PUT, DELETE methods
    """
    def __init__(self, token: str, configuration_type_id: str):
        self.configuration_type_id = configuration_type_id
        super().__init__(token)

    def get_endpoint(self):
        return f'configurationtypes/{self.configuration_type_id}'


class APICallConfigTypeServices(APIEndPointAuthenticated):
    """
        Implement GET method
    """
    def __init__(self, token: str, configuration_type_id: str):
        self.configuration_type_id = configuration_type_id
        super().__init__(token)

    def get_endpoint(self):
        return f'configurationtypes/{self.configuration_type_id}/services'


class APICallConfigTypeServiceConfig(APIEndPointAuthenticated):
    """
        Implement GET, PUT methods
    """
    def __init__(self, token: str, configuration_type_id: str, service_configuration_id: str):
        self.configuration_type_id = configuration_type_id
        self.service_configuration_id = service_configuration_id
        super().__init__(token)

    def get_endpoint(self):
        return f'configurationtypes/{self.configuration_type_id}/services/{self.service_configuration_id}'


def get_project_config_types(token, project_id):
    return APICallProjectConfigTypes(token, project_id).get()


def post_project_config_types(token, project_id, data):
    return APICallProjectConfigTypes(token, project_id).post(data)


def get_config_type(token, configuration_type_id):
    return APICallConfigType(token, configuration_type_id).get()


def put_config_type(token, configuration_type_id, data):
    return APICallConfigType(token, configuration_type_id).put(data)


def delete_config_type(token, configuration_type_id):
    return APICallConfigType(token, configuration_type_id).delete()


def get_config_type_services(token, configuration_type_id):
    return APICallConfigTypeServices(token, configuration_type_id).get()


def get_config_type_service_config(token, configuration_type_id, service_configuration_id):
    return APICallConfigTypeServiceConfig(token, configuration_type_id, service_configuration_id).get()


def put_config_type_service_config(token, configuration_type_id, service_configuration_id, data):
    return APICallConfigTypeServiceConfig(token, configuration_type_id, service_configuration_id).put(data)



from robotcloud.api import APIEndPointAuthenticated
from robotcloud.exceptions import BadUsageException


class APICallLocation(APIEndPointAuthenticated):
    def __init__(self, token: str, location_id: str):
        self.location_id = location_id
        super().__init__(token)

    def get_endpoint(self):
        return f"locations/{self.location_id}"


class ProjectLocationsEndpoint(APIEndPointAuthenticated):
    def __init__(self, token: str, project_id: str):
        self.project_id = project_id
        super().__init__(token)

    def get_endpoint(self):
        return f"projects/{self.project_id}/locations"


def create_location(token, data: dict):
    if 'project_id' not in data or not isinstance(data['project_id'], str):
        raise BadUsageException("")

    project_id = data['project_id']
    data.pop('project_id')
    return ProjectLocationsEndpoint(token, project_id).post(data)


def get_project_locations(token, project_id):
    return ProjectLocationsEndpoint(token, project_id).get()


def get_location(token, location_id):
    return APICallLocation(token, location_id).get()


def update_location(token, location_id, data):
    return APICallLocation(token, location_id).put(data)


def delete_location(token, location_id):
    return APICallLocation(token, location_id).delete()



from dataclasses import dataclass
from typing import List


@dataclass
class PaginatedResponse:
    data: List
    total_records: int  # Total records, before filtering
    total_records_filtered: int  # Total records, after filtering



#!/usr/bin/env python
import os
import sys
from codecs import open

from setuptools import setup, find_packages

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 8)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write(
        """
==========================
Unsupported Python version
==========================
This version of Requests requires at least Python {}.{}, but
you're trying to install it on Python {}.{}. To resolve this,
consider upgrading to a supported Python version.
If you can't upgrade your Python version, you'll need to
pin to an older version of Requests (<2.28).
""".format(
            *(REQUIRED_PYTHON + CURRENT_PYTHON)
        )
    )
    sys.exit(1)


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='robotcloud',
    version='0.1.1',
    author="Bernat Galmés Rubert",
    author_email="bernat.galmes@robotbas.com",
    description="Client to interact with robotcloud API from a python project.",
    url='https://github.com/robotmallorca/sw.module.python.robotcloud.client',
    keywords="client robotcloud",
    packages=find_packages(include=['robotcloud', 'robotcloud.endpoints', 'robotcloud.utils']),
    long_description=read('README.md'),
    install_requires=[
        'python>=3.8.0',
        'requests>=2.28'
    ],
    setup_requires=['flake8']
)



from robotcloud.api import APIEndPointAuthenticated, APIEndPointUnAuthenticated


class LoginEndpoint(APIEndPointUnAuthenticated):
    def __init__(self, username: str, password: str):
        super().__init__(username, password)

    def get_endpoint(self):
        return "login"


class LoginRenewEndpoint(APIEndPointAuthenticated):
    def __init__(self, token: str):
        super().__init__(token)

    def get_endpoint(self):
        return "login/renew"


def login(username: str, password: str):
    return LoginEndpoint(username, password).get()


def login_renew(token):
    return LoginRenewEndpoint(token).get()
